@@ -4,7 +4,7 @@ set -e
 test -d m4 || mkdir m4
 gtkdocize || exit 1
 
-autoreconf -i
+autoreconf -fi
 
 run_configure=true
 for arg in $*; do
