@@ -54,11 +54,9 @@
 #endif
 
 typedef struct {
-  NiceAgent *agent;
   NiceAddress server_addr;
   GQueue send_queue;
   GMainContext *context;
-  GIOChannel *io_channel;
   GSource *io_source;
   gboolean error;
 } TcpPriv;
@@ -82,18 +80,19 @@ static gboolean socket_is_reliable (NiceSocket *sock);
 static void add_to_be_sent (NiceSocket *sock, const gchar *buf, guint len,
     gboolean head);
 static void free_to_be_sent (struct to_be_sent *tbs);
-static gboolean socket_send_more (GIOChannel *source, GIOCondition condition,
+static gboolean socket_send_more (GSocket *gsocket, GIOCondition condition,
     gpointer data);
 
 NiceSocket *
-nice_tcp_bsd_socket_new (NiceAgent *agent, GMainContext *ctx, NiceAddress *addr)
+nice_tcp_bsd_socket_new (GMainContext *ctx, NiceAddress *addr)
 {
-  int sockfd = -1;
-  int ret;
   struct sockaddr_storage name;
-  guint name_len = sizeof (name);
   NiceSocket *sock;
   TcpPriv *priv;
+  GSocket *gsock = NULL;
+  GError *gerr = NULL;
+  gboolean gret = FALSE;
+  GSocketAddress *gaddr;
 
   if (addr == NULL) {
     /* We can't connect a tcp socket with no destination address */
@@ -102,17 +101,20 @@ nice_tcp_bsd_socket_new (NiceAgent *agent, GMainContext *ctx, NiceAddress *addr)
 
   sock = g_slice_new0 (NiceSocket);
 
-  nice_address_copy_to_sockaddr(addr, (struct sockaddr *)&name);
+  nice_address_copy_to_sockaddr (addr, (struct sockaddr *)&name);
 
-  if (sockfd == -1) {
+  if (gsock == NULL) {
     if (name.ss_family == AF_UNSPEC || name.ss_family == AF_INET) {
-      sockfd = socket (PF_INET, SOCK_STREAM, 0);
+      gsock = g_socket_new (G_SOCKET_FAMILY_IPV4, G_SOCKET_TYPE_STREAM,
+          G_SOCKET_PROTOCOL_TCP, NULL);
+
       name.ss_family = AF_INET;
 #ifdef HAVE_SA_LEN
       name.ss_len = sizeof (struct sockaddr_in);
 #endif
     } else if (name.ss_family == AF_INET6) {
-      sockfd = socket (PF_INET6, SOCK_STREAM, 0);
+      gsock = g_socket_new (G_SOCKET_FAMILY_IPV6, G_SOCKET_TYPE_STREAM,
+          G_SOCKET_PROTOCOL_TCP, NULL);
       name.ss_family = AF_INET6;
 #ifdef HAVE_SA_LEN
       name.ss_len = sizeof (struct sockaddr_in6);
@@ -120,55 +122,50 @@ nice_tcp_bsd_socket_new (NiceAgent *agent, GMainContext *ctx, NiceAddress *addr)
     }
   }
 
-  if (sockfd == -1) {
+  if (gsock == NULL) {
     g_slice_free (NiceSocket, sock);
     return NULL;
   }
 
-#ifdef FD_CLOEXEC
-  fcntl (sockfd, F_SETFD, fcntl (sockfd, F_GETFD) | FD_CLOEXEC);
-#endif
-#ifdef O_NONBLOCK
-  fcntl (sockfd, F_SETFL, fcntl (sockfd, F_GETFL) | O_NONBLOCK);
-#endif
+  /* GSocket: All socket file descriptors are set to be close-on-exec. */
+  g_socket_set_blocking (gsock, false);
 
-  name_len = name.ss_family == AF_INET? sizeof (struct sockaddr_in) :
-      sizeof(struct sockaddr_in6);
-  ret = connect (sockfd, (const struct sockaddr *)&name, name_len);
+  gaddr = g_socket_address_new_from_native (&name, sizeof (name));
 
-#ifdef G_OS_WIN32
-  if (ret < 0 && WSAGetLastError () != WSAEINPROGRESS) {
-    closesocket (sockfd);
-#else
-  if (ret < 0 && errno != EINPROGRESS) {
-    close (sockfd);
-#endif
-    g_slice_free (NiceSocket, sock);
-    return NULL;
+  if (gaddr != NULL) {
+    gret = g_socket_connect (gsock, gaddr, NULL, &gerr);
+    g_object_unref (gaddr);
   }
 
-  name_len = name.ss_family == AF_INET? sizeof (struct sockaddr_in) :
-      sizeof(struct sockaddr_in6);
-  if (getsockname (sockfd, (struct sockaddr *) &name, &name_len) < 0) {
+  if (gret == FALSE) {
+    if (g_error_matches (gerr, G_IO_ERROR, G_IO_ERROR_PENDING) == FALSE) {
+      g_socket_close (gsock, NULL);
+      g_object_unref (gsock);
+      g_slice_free (NiceSocket, sock);
+      return NULL;
+    }
+    g_error_free(gerr);
+  }
+
+  gaddr = g_socket_get_local_address (gsock, NULL);
+  if (gaddr == NULL ||
+      !g_socket_address_to_native (gaddr, &name, sizeof (name), NULL)) {
     g_slice_free (NiceSocket, sock);
-#ifdef G_OS_WIN32
-    closesocket(sockfd);
-#else
-    close (sockfd);
-#endif
+    g_socket_close (gsock, NULL);
+    g_object_unref (gsock);
     return NULL;
   }
+  g_object_unref (gaddr);
 
   nice_address_set_from_sockaddr (&sock->addr, (struct sockaddr *)&name);
 
   sock->priv = priv = g_slice_new0 (TcpPriv);
 
-  priv->agent = agent;
-  priv->context = ctx;
+  priv->context = g_main_context_ref (ctx);
   priv->server_addr = *addr;
   priv->error = FALSE;
 
-  sock->fileno = sockfd;
+  sock->fileno = gsock;
   sock->send = socket_send;
   sock->recv = socket_recv;
   sock->is_reliable = socket_is_reliable;
@@ -183,20 +180,21 @@ socket_close (NiceSocket *sock)
 {
   TcpPriv *priv = sock->priv;
 
-#ifdef G_OS_WIN32
-  closesocket(sock->fileno);
-#else
-  close (sock->fileno);
-#endif
+  if (sock->fileno) {
+    g_socket_close (sock->fileno, NULL);
+    g_object_unref (sock->fileno);
+    sock->fileno = NULL;
+  }
   if (priv->io_source) {
     g_source_destroy (priv->io_source);
     g_source_unref (priv->io_source);
   }
-  if (priv->io_channel)
-    g_io_channel_unref (priv->io_channel);
   g_queue_foreach (&priv->send_queue, (GFunc) free_to_be_sent, NULL);
   g_queue_clear (&priv->send_queue);
 
+  if (priv->context)
+    g_main_context_unref (priv->context);
+
   g_slice_free(TcpPriv, sock->priv);
 }
 
@@ -205,12 +203,13 @@ socket_recv (NiceSocket *sock, NiceAddress *from, guint len, gchar *buf)
 {
   TcpPriv *priv = sock->priv;
   int ret;
+  GError *gerr = NULL;
 
   /* Don't try to access the socket if it had an error */
   if (priv->error)
     return -1;
 
-  ret = recv (sock->fileno, buf, len, 0);
+  ret = g_socket_receive (sock->fileno, buf, len, NULL, &gerr);
 
   /* recv returns 0 when the peer performed a shutdown.. we must return -1 here
    * so that the agent destroys the g_source */
@@ -220,14 +219,11 @@ socket_recv (NiceSocket *sock, NiceAddress *from, guint len, gchar *buf)
   }
 
   if (ret < 0) {
-#ifdef G_OS_WIN32
-    if (WSAGetLastError () == WSAEWOULDBLOCK)
-#else
-    if (errno == EAGAIN)
-#endif
-      return 0;
-    else
-      return ret;
+    if(g_error_matches (gerr, G_IO_ERROR, G_IO_ERROR_WOULD_BLOCK))
+      ret = 0;
+
+    g_error_free (gerr);
+    return ret;
   }
 
   if (from)
@@ -244,6 +240,7 @@ socket_send (NiceSocket *sock, const NiceAddress *to,
 {
   TcpPriv *priv = sock->priv;
   int ret;
+  GError *gerr = NULL;
 
   /* Don't try to access the socket if it had an error, otherwise we risk a
      crash with SIGPIPE (Broken pipe) */
@@ -253,17 +250,15 @@ socket_send (NiceSocket *sock, const NiceAddress *to,
   /* First try to send the data, don't send it later if it can be sent now
      this way we avoid allocating memory on every send */
   if (g_queue_is_empty (&priv->send_queue)) {
-    ret = send (sock->fileno, buf, len, 0);
-
+    ret = g_socket_send (sock->fileno, buf, len, NULL, &gerr);
     if (ret < 0) {
-#ifdef G_OS_WIN32
-      if (WSAGetLastError () == WSAEWOULDBLOCK) {
-#else
-      if (errno == EAGAIN) {
-#endif
+      if(g_error_matches (gerr, G_IO_ERROR, G_IO_ERROR_WOULD_BLOCK)
+         || g_error_matches (gerr, G_IO_ERROR, G_IO_ERROR_FAILED)) {
         add_to_be_sent (sock, buf, len, FALSE);
+        g_error_free (gerr);
         return TRUE;
       } else {
+        g_error_free (gerr);
         return FALSE;
       }
     } else if ((guint)ret < len) {
@@ -307,13 +302,14 @@ socket_is_reliable (NiceSocket *sock)
 
 static gboolean
 socket_send_more (
-  GIOChannel *source,
+  GSocket *gsocket,
   GIOCondition condition,
   gpointer data)
 {
   NiceSocket *sock = (NiceSocket *) data;
   TcpPriv *priv = sock->priv;
   struct to_be_sent *tbs = NULL;
+  GError *gerr = NULL;
 
   agent_lock ();
 
@@ -327,19 +323,23 @@ socket_send_more (
   while ((tbs = g_queue_pop_head (&priv->send_queue)) != NULL) {
     int ret;
 
-    ret = send (sock->fileno, tbs->buf, tbs->length, 0);
+    if(condition & G_IO_HUP) {
+      /* connection hangs up */
+      ret = -1;
+    } else {
+      ret = g_socket_send (sock->fileno, tbs->buf, tbs->length, NULL, &gerr);
+    }
 
     if (ret < 0) {
-#ifdef G_OS_WIN32
-      if (WSAGetLastError () == WSAEWOULDBLOCK) {
-#else
-      if (errno == EAGAIN) {
-#endif
+      if(gerr != NULL &&
+          g_error_matches (gerr, G_IO_ERROR, G_IO_ERROR_WOULD_BLOCK)) {
         add_to_be_sent (sock, tbs->buf, tbs->length, TRUE);
         g_free (tbs->buf);
         g_slice_free (struct to_be_sent, tbs);
+        g_error_free (gerr);
         break;
       }
+      g_error_free (gerr);
     } else if (ret < (int) tbs->length) {
       add_to_be_sent (sock, tbs->buf + ret, tbs->length - ret, TRUE);
       g_free (tbs->buf);
@@ -352,8 +352,6 @@ socket_send_more (
   }
 
   if (g_queue_is_empty (&priv->send_queue)) {
-    g_io_channel_unref (priv->io_channel);
-    priv->io_channel = NULL;
     g_source_destroy (priv->io_source);
     g_source_unref (priv->io_source);
     priv->io_source = NULL;
@@ -385,9 +383,8 @@ add_to_be_sent (NiceSocket *sock, const gchar *buf, guint len, gboolean head)
   else
     g_queue_push_tail (&priv->send_queue, tbs);
 
-  if (priv->io_channel == NULL) {
-    priv->io_channel = g_io_channel_unix_new (sock->fileno);
-    priv->io_source = g_io_create_watch (priv->io_channel, G_IO_OUT);
+  if (priv->io_source == NULL) {
+    priv->io_source = g_socket_create_source(sock->fileno, G_IO_OUT, NULL);
     g_source_set_callback (priv->io_source, (GSourceFunc) socket_send_more,
         sock, NULL);
     g_source_attach (priv->io_source, priv->context);
