@@ -70,7 +70,7 @@ static gboolean socket_send (NiceSocket *sock, const NiceAddress *to,
 static gboolean socket_is_reliable (NiceSocket *sock);
 
 NiceSocket *
-nice_tcp_turn_socket_new (NiceAgent *agent, NiceSocket *base_socket,
+nice_tcp_turn_socket_new (NiceSocket *base_socket,
     NiceTurnSocketCompatibility compatibility)
 {
   TurnTcpPriv *priv;
@@ -152,7 +152,7 @@ socket_recv (NiceSocket *sock, NiceAddress *from, guint len, gchar *buf)
   }
 
   if (priv->compatibility == NICE_TURN_SOCKET_COMPATIBILITY_DRAFT9 ||
-        priv->compatibility == NICE_TURN_SOCKET_COMPATIBILITY_RFC5766)
+      priv->compatibility == NICE_TURN_SOCKET_COMPATIBILITY_RFC5766)
     padlen = (priv->expecting_len % 4) ?  4 - (priv->expecting_len % 4) : 0;
   else
     padlen = 0;
@@ -188,8 +188,8 @@ socket_send (NiceSocket *sock, const NiceAddress *to,
   gchar buffer[MAX_UDP_MESSAGE_SIZE + sizeof(guint16) + sizeof(padbuf)];
   guint buffer_len = 0;
 
-  if (priv->compatibility != NICE_TURN_SOCKET_COMPATIBILITY_DRAFT9 ||
-        priv->compatibility == NICE_TURN_SOCKET_COMPATIBILITY_RFC5766)
+  if (priv->compatibility != NICE_TURN_SOCKET_COMPATIBILITY_DRAFT9 &&
+      priv->compatibility != NICE_TURN_SOCKET_COMPATIBILITY_RFC5766)
     padlen = 0;
 
   if (priv->compatibility == NICE_TURN_SOCKET_COMPATIBILITY_GOOGLE) {
@@ -202,7 +202,7 @@ socket_send (NiceSocket *sock, const NiceAddress *to,
   buffer_len += len;
 
   if (priv->compatibility == NICE_TURN_SOCKET_COMPATIBILITY_DRAFT9 ||
-        priv->compatibility == NICE_TURN_SOCKET_COMPATIBILITY_RFC5766) {
+      priv->compatibility == NICE_TURN_SOCKET_COMPATIBILITY_RFC5766) {
     memcpy (buffer + buffer_len, padbuf, padlen);
     buffer_len += padlen;
   }
