@@ -47,7 +47,6 @@ typedef enum {
 } NiceTurnSocketCompatibility;
 
 #include "socket.h"
-#include "agent.h"
 #include "stun/stunmessage.h"
 
 
@@ -62,7 +61,7 @@ gboolean
 nice_turn_socket_set_peer (NiceSocket *sock, NiceAddress *peer);
 
 NiceSocket *
-nice_turn_socket_new (NiceAgent *agent, NiceAddress *addr,
+nice_turn_socket_new (GMainContext *ctx, NiceAddress *addr,
     NiceSocket *base_socket, NiceAddress *server_addr,
     gchar *username, gchar *password, NiceTurnSocketCompatibility compatibility);
 
