@@ -79,7 +79,8 @@ size_t stun_usage_turn_create (StunAgent *agent, StunMessage *msg,
 {
   stun_agent_init_request (agent, msg, buffer, buffer_len, STUN_ALLOCATE);
 
-  if (compatibility == STUN_USAGE_TURN_COMPATIBILITY_DRAFT9) {
+  if (compatibility == STUN_USAGE_TURN_COMPATIBILITY_DRAFT9 ||
+      compatibility == STUN_USAGE_TURN_COMPATIBILITY_RFC5766) {
     if (stun_message_append32 (msg, STUN_ATTRIBUTE_REQUESTED_TRANSPORT,
             TURN_REQUESTED_TRANSPORT_UDP) != STUN_MESSAGE_RETURN_SUCCESS)
       return 0;
@@ -94,13 +95,18 @@ size_t stun_usage_turn_create (StunAgent *agent, StunMessage *msg,
       return 0;
   }
 
+  if (compatibility == STUN_USAGE_TURN_COMPATIBILITY_OC2007) {
+    stun_message_append32(msg, STUN_ATTRIBUTE_MS_VERSION, 1);
+  }
+
   if (lifetime >= 0) {
     if (stun_message_append32 (msg, STUN_ATTRIBUTE_LIFETIME, lifetime) !=
         STUN_MESSAGE_RETURN_SUCCESS)
       return 0;
   }
 
-  if (compatibility == STUN_USAGE_TURN_COMPATIBILITY_DRAFT9 &&
+  if ((compatibility == STUN_USAGE_TURN_COMPATIBILITY_DRAFT9 ||
+          compatibility == STUN_USAGE_TURN_COMPATIBILITY_RFC5766) &&
       request_props != STUN_USAGE_TURN_REQUEST_PORT_NORMAL) {
     uint32_t req = 0;
 
@@ -137,7 +143,8 @@ size_t stun_usage_turn_create (StunAgent *agent, StunMessage *msg,
           STUN_MESSAGE_RETURN_SUCCESS)
         return 0;
     }
-    if (stun_message_find64 (previous_response, STUN_ATTRIBUTE_RESERVATION_TOKEN,
+    if (stun_message_find64 (previous_response,
+            STUN_ATTRIBUTE_RESERVATION_TOKEN,
             &reservation) == STUN_MESSAGE_RETURN_SUCCESS) {
       if (stun_message_append64 (msg, STUN_ATTRIBUTE_RESERVATION_TOKEN,
               reservation) != STUN_MESSAGE_RETURN_SUCCESS)
@@ -162,7 +169,8 @@ size_t stun_usage_turn_create_refresh (StunAgent *agent, StunMessage *msg,
     StunUsageTurnCompatibility compatibility)
 {
 
-  if (compatibility != STUN_USAGE_TURN_COMPATIBILITY_DRAFT9) {
+  if (compatibility != STUN_USAGE_TURN_COMPATIBILITY_DRAFT9 &&
+      compatibility != STUN_USAGE_TURN_COMPATIBILITY_RFC5766) {
     return stun_usage_turn_create (agent, msg, buffer, buffer_len,
         previous_response, STUN_USAGE_TURN_REQUEST_PORT_NORMAL, 0, lifetime,
         username, username_len, password, password_len, compatibility);
@@ -207,6 +215,52 @@ size_t stun_usage_turn_create_refresh (StunAgent *agent, StunMessage *msg,
   return stun_agent_finish_message (agent, msg, password, password_len);
 }
 
+size_t stun_usage_turn_create_permission (StunAgent *agent, StunMessage *msg,
+    uint8_t *buffer, size_t buffer_len,
+    uint8_t *username, size_t username_len,
+    uint8_t *password, size_t password_len,
+    uint8_t *realm, size_t realm_len,
+    uint8_t *nonce, size_t nonce_len,
+    struct sockaddr *peer,
+    StunUsageTurnCompatibility compatibility)
+{
+  if (!peer)
+    return 0;
+
+  stun_agent_init_request (agent, msg, buffer, buffer_len,
+      STUN_CREATEPERMISSION);
+
+  /* PEER address */
+  if (stun_message_append_xor_addr (msg, STUN_ATTRIBUTE_XOR_PEER_ADDRESS,
+          peer, sizeof(*peer)) != STUN_MESSAGE_RETURN_SUCCESS) {
+    return 0;
+  }
+
+  /* nonce */
+  if (nonce != NULL) {
+    if (stun_message_append_bytes (msg, STUN_ATTRIBUTE_NONCE,
+            nonce, nonce_len) != STUN_MESSAGE_RETURN_SUCCESS)
+      return 0;
+  }
+
+  /* realm */
+  if (realm != NULL) {
+    if (stun_message_append_bytes (msg, STUN_ATTRIBUTE_REALM,
+            realm, realm_len) != STUN_MESSAGE_RETURN_SUCCESS)
+      return 0;
+  }
+
+  /* username */
+  if (username != NULL) {
+    if (stun_message_append_bytes (msg, STUN_ATTRIBUTE_USERNAME,
+            username, username_len) != STUN_MESSAGE_RETURN_SUCCESS)
+      return 0;
+  }
+
+  return stun_agent_finish_message (agent, msg, password, password_len);
+}
+
+
 StunUsageTurnReturn stun_usage_turn_process (StunMessage *msg,
     struct sockaddr *relay_addr, socklen_t *relay_addrlen,
     struct sockaddr *addr, socklen_t *addrlen,
@@ -265,7 +319,8 @@ StunUsageTurnReturn stun_usage_turn_process (StunMessage *msg,
 
   stun_debug ("Received %u-bytes STUN message\n", stun_message_length (msg));
 
-  if (compatibility == STUN_USAGE_TURN_COMPATIBILITY_DRAFT9) {
+  if (compatibility == STUN_USAGE_TURN_COMPATIBILITY_DRAFT9 ||
+      compatibility == STUN_USAGE_TURN_COMPATIBILITY_RFC5766) {
     val = stun_message_find_xor_addr (msg,
         STUN_ATTRIBUTE_XOR_MAPPED_ADDRESS, addr, addrlen);
 
@@ -284,7 +339,8 @@ StunUsageTurnReturn stun_usage_turn_process (StunMessage *msg,
       stun_debug (" No MAPPED-ADDRESS: %d\n", val);
       return STUN_USAGE_TURN_RETURN_ERROR;
     }
-  } else if (compatibility == STUN_USAGE_TURN_COMPATIBILITY_MSN) {
+  } else if (compatibility == STUN_USAGE_TURN_COMPATIBILITY_MSN ||
+      compatibility == STUN_USAGE_TURN_COMPATIBILITY_OC2007) {
     val = stun_message_find_addr (msg,
         STUN_ATTRIBUTE_MSN_MAPPED_ADDRESS, addr, addrlen);
 
@@ -315,7 +371,8 @@ StunUsageTurnReturn stun_usage_turn_refresh_process (StunMessage *msg,
   int code = -1;
   StunUsageTurnReturn ret = STUN_USAGE_TURN_RETURN_RELAY_SUCCESS;
 
-  if (compatibility == STUN_USAGE_TURN_COMPATIBILITY_DRAFT9) {
+  if (compatibility == STUN_USAGE_TURN_COMPATIBILITY_DRAFT9 ||
+      compatibility == STUN_USAGE_TURN_COMPATIBILITY_RFC5766) {
     if (stun_message_get_method (msg) != STUN_REFRESH)
       return STUN_USAGE_TURN_RETURN_INVALID;
   } else {
