@@ -202,7 +202,7 @@ static void cb_candidate_gathering_done(NiceAgent *agent, guint stream_id, gpoin
 
 static void cb_component_state_changed (NiceAgent *agent, guint stream_id, guint component_id, guint state, gpointer data)
 {
-  g_debug ("test-fullmode:%s: %p", __func__, data);
+  g_debug ("test-fullmode:%s: %p", G_STRFUNC, data);
 
   if (GPOINTER_TO_UINT (data) == 1)
     global_lagent_state[component_id - 1] = state;
@@ -240,7 +240,7 @@ static void cb_component_state_changed (NiceAgent *agent, guint stream_id, guint
 static void cb_new_selected_pair(NiceAgent *agent, guint stream_id, guint component_id, 
 				 gchar *lfoundation, gchar* rfoundation, gpointer data)
 {
-  g_debug ("test-fullmode:%s: %p", __func__, data);
+  g_debug ("test-fullmode:%s: %p", G_STRFUNC, data);
 
   if (GPOINTER_TO_UINT (data) == 1)
     ++global_lagent_cands;
@@ -254,7 +254,7 @@ static void cb_new_selected_pair(NiceAgent *agent, guint stream_id, guint compon
 static void cb_new_candidate(NiceAgent *agent, guint stream_id, guint component_id, 
 			     gchar *foundation, gpointer data)
 {
-  g_debug ("test-fullmode:%s: %p", __func__, data);
+  g_debug ("test-fullmode:%s: %p", G_STRFUNC, data);
 
   /* XXX: dear compiler, these are for you: */
   (void)agent; (void)stream_id; (void)data; (void)component_id; (void)foundation;
@@ -262,7 +262,7 @@ static void cb_new_candidate(NiceAgent *agent, guint stream_id, guint component_
 
 static void cb_initial_binding_request_received(NiceAgent *agent, guint stream_id, gpointer data)
 {
-  g_debug ("test-fullmode:%s: %p", __func__, data);
+  g_debug ("test-fullmode:%s: %p", G_STRFUNC, data);
 
   if (GPOINTER_TO_UINT (data) == 1)
     global_lagent_ibr_received = TRUE;
@@ -803,8 +803,16 @@ int main (void)
   guint timer_id;
   const char *stun_server = NULL, *stun_server_port = NULL;
 
+#ifdef G_OS_WIN32
+  WSADATA w;
+
+  WSAStartup(0x0202, &w);
+#endif
   g_type_init ();
-  g_thread_init (NULL);
+#if !GLIB_CHECK_VERSION(2,31,8)
+  g_thread_init(NULL);
+#endif
+
   global_mainloop = g_main_loop_new (NULL, FALSE);
 
   /* Note: impl limits ...
@@ -881,6 +889,7 @@ int main (void)
   g_object_set (G_OBJECT (lagent), "proxy-type", PROXY_TYPE, NULL);
   g_object_set (G_OBJECT (lagent), "proxy-username", PROXY_USERNAME, NULL);
   g_object_set (G_OBJECT (lagent), "proxy-password", PROXY_PASSWORD, NULL);
+  g_object_set (G_OBJECT (ragent), "upnp", USE_UPNP,  NULL);
   g_object_set (G_OBJECT (ragent), "proxy-ip", PROXY_IP,  NULL);
   g_object_set (G_OBJECT (ragent), "proxy-port", PROXY_PORT, NULL);
   g_object_set (G_OBJECT (ragent), "proxy-type", PROXY_TYPE, NULL);
@@ -1042,6 +1051,8 @@ int main (void)
   global_mainloop = NULL;
 
   g_source_remove (timer_id);
-
+#ifdef G_OS_WIN32
+  WSACleanup();
+#endif
   return result;
 }
