@@ -41,6 +41,9 @@
 
 #include <stdlib.h>
 #include <string.h>
+#ifdef _WIN32
+#include <io.h>
+#endif
 
 static NiceComponentState global_lagent_state = NICE_COMPONENT_STATE_LAST;
 static NiceComponentState global_ragent_state = NICE_COMPONENT_STATE_LAST;
@@ -111,7 +114,7 @@ static void cb_candidate_gathering_done(NiceAgent *agent, guint stream_id, gpoin
 
 static void cb_component_state_changed (NiceAgent *agent, guint stream_id, guint component_id, guint state, gpointer data)
 {
-  g_debug ("test-restart:%s: %p", __func__, data);
+  g_debug ("test-restart:%s: %p", G_STRFUNC, data);
 
   if ((intptr_t)data == 1)
     global_lagent_state = state;
@@ -144,7 +147,7 @@ static void cb_component_state_changed (NiceAgent *agent, guint stream_id, guint
 static void cb_new_selected_pair(NiceAgent *agent, guint stream_id, guint component_id, 
 				 gchar *lfoundation, gchar* rfoundation, gpointer data)
 {
-  g_debug ("test-restart:%s: %p", __func__, data);
+  g_debug ("test-restart:%s: %p", G_STRFUNC, data);
 
   if ((intptr_t)data == 1)
     ++global_lagent_cands;
@@ -158,7 +161,7 @@ static void cb_new_selected_pair(NiceAgent *agent, guint stream_id, guint compon
 static void cb_new_candidate(NiceAgent *agent, guint stream_id, guint component_id, 
 			     gchar *foundation, gpointer data)
 {
-  g_debug ("test-restart:%s: %p", __func__, data);
+  g_debug ("test-restart:%s: %p", G_STRFUNC, data);
 
   /* XXX: dear compiler, these are for you: */
   (void)agent; (void)stream_id; (void)data; (void)component_id; (void)foundation;
@@ -166,7 +169,7 @@ static void cb_new_candidate(NiceAgent *agent, guint stream_id, guint component_
 
 static void cb_initial_binding_request_received(NiceAgent *agent, guint stream_id, gpointer data)
 {
-  g_debug ("test-restart:%s: %p", __func__, data);
+  g_debug ("test-restart:%s: %p", G_STRFUNC, data);
 
   if ((intptr_t)data == 1)
     global_lagent_ibr_received = TRUE;
@@ -393,8 +396,16 @@ int main (void)
   guint timer_id;
   const char *stun_server = NULL, *stun_server_port = NULL;
 
+#ifdef G_OS_WIN32
+  WSADATA w;
+
+  WSAStartup(0x0202, &w);
+#endif
   g_type_init ();
-  g_thread_init (NULL);
+#if !GLIB_CHECK_VERSION(2,31,8)
+  g_thread_init(NULL);
+#endif
+
   global_mainloop = g_main_loop_new (NULL, FALSE);
 
   /* Note: impl limits ...
@@ -463,6 +474,8 @@ int main (void)
   global_mainloop = NULL;
 
   g_source_remove (timer_id);
-
+#ifdef G_OS_WIN32
+  WSACleanup();
+#endif
   return result;
 }
